/**
 * <copyright>
 * 
 * Copyright (c) 2013 itemis and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     itemis - Initial API and implementation
 * 
 * </copyright>
 */
package org.safe.ide.preferences;

import java.util.Collection;

import org.eclipse.core.resources.IProject;
import org.eclipse.sphinx.emf.metamodel.IMetaModelDescriptor;
import org.eclipse.sphinx.emf.metamodel.MetaModelDescriptorRegistry;
import org.eclipse.sphinx.platform.preferences.AbstractProjectPreference;
import org.eclipse.sphinx.platform.preferences.AbstractProjectWorkspacePreference;
import org.eclipse.sphinx.platform.preferences.IProjectPreference;
import org.eclipse.sphinx.platform.preferences.IProjectWorkspacePreference;
import org.safe.common.metamodel.SAFEReleaseDescriptor;
import org.safe.common.preferences.ISAFEPreferenceConstants;
import org.safe.ide.internal.Activator;
import org.safe.ide.natures.SAFENature;

/**
 * A set of preferences through which users can adjust the way how they work with SAFE models in the workspace.
 */
public interface ISAFEWorkspacePreferences {

	/**
	 * The {@link SAFEReleaseDescriptor SAFE release} to be used. Can be defined globally at workspace level and may be
	 * individually redefined, i.e., overridden, by each project.
	 */
	IProjectWorkspacePreference<SAFEReleaseDescriptor> SAFE_RELEASE = new AbstractProjectWorkspacePreference<SAFEReleaseDescriptor>(SAFENature.ID,
			Activator.getPlugin().getSymbolicName(), ISAFEWorkspacePreferenceConstants.PREF_SAFE_RELEASE,
			ISAFEWorkspacePreferenceConstants.PREF_SAFE_RELEASE_DEFAULT) {
		/*
		 * @see org.eclipse.sphinx.platform.preferences. AbstractProjectWorkspacePreference
		 * #toObject(org.eclipse.core.resources .IProject, java.lang.String)
		 */
		@Override
		protected SAFEReleaseDescriptor toObject(final IProject project, String valueAsString) {
			// Assume that given value string is the id of an SAFE release
			// descriptor and try to retrieve the latter
			IMetaModelDescriptor mmDescriptor = MetaModelDescriptorRegistry.INSTANCE.getDescriptor(valueAsString);
			if (mmDescriptor instanceof SAFEReleaseDescriptor) {
				return (SAFEReleaseDescriptor) mmDescriptor;
			}

			// Go for the SAFE release
			// preference default
			return null;
		}

		/*
		 * @see org.eclipse.sphinx.platform.preferences. AbstractProjectWorkspacePreference#toString(java.lang.Object)
		 */
		@Override
		protected String toString(SAFEReleaseDescriptor valueAsObject) {
			return valueAsObject != null ? valueAsObject.getIdentifier() : null;
		}
	};

	/**
	 * The {@link SAFEReleaseDescriptor SAFE resource version} to be used for serializing the resources of an SAFE
	 * model. The {@link SAFEReleaseDescriptor SAFE resource version} may be the same or older than the release of the
	 * metamodel implementation used for manipulating the SAFE model in question. Can be defined individually by each
	 * project.
	 */
	IProjectPreference<SAFEReleaseDescriptor> RESOURCE_VERSION = new AbstractProjectPreference<SAFEReleaseDescriptor>(SAFENature.ID, Activator
			.getPlugin().getSymbolicName(), ISAFEPreferenceConstants.PREF_SAFE_RESOURCE_VERSION_DEFAULT,
			ISAFEPreferenceConstants.PREF_SAFE_RESOURCE_VERSION_DEFAULT) {

		/*
		 * @see org.eclipse.sphinx.platform.preferences.AbstractProjectPreference
		 * #toObject(org.eclipse.core.resources.IProject , java.lang.String)
		 */
		@Override
		protected SAFEReleaseDescriptor toObject(IProject project, String valueAsString) {
			if (ISAFEPreferenceConstants.PREF_SAFE_RESOURCE_VERSION_DEFAULT.equals(valueAsString)) {
				return null;
			}

			SAFEReleaseDescriptor safeRelease = SAFE_RELEASE.get(project);
			if (safeRelease.getIdentifier().equals(valueAsString)) {
				return safeRelease;
			}
			Collection<IMetaModelDescriptor> resourceVersions = safeRelease.getCompatibleResourceVersionDescriptors();
			for (IMetaModelDescriptor resourceVersion : resourceVersions) {
				if (resourceVersion instanceof SAFEReleaseDescriptor && resourceVersion.getIdentifier().equals(valueAsString)) {
					return (SAFEReleaseDescriptor) resourceVersion;
				}
			}

			throw new RuntimeException("Resource version " //$NON-NLS-1$
					+ valueAsString + " is not compatible with SAFE release " + safeRelease.getName() //$NON-NLS-1$
					+ " set in project " //$NON-NLS-1$
					+ project.getName() + "."); //$NON-NLS-1$
		}

		/*
		 * @see org.eclipse.sphinx.platform.preferences.AbstractProjectPreference
		 * #toString(org.eclipse.core.resources.IProject , java.lang.Object)
		 */
		@Override
		protected String toString(IProject project, SAFEReleaseDescriptor valueAsObject) {
			if (valueAsObject == null) {
				return ISAFEPreferenceConstants.PREF_SAFE_RESOURCE_VERSION_DEFAULT;
			}

			SAFEReleaseDescriptor safeRelease = SAFE_RELEASE.get(project);
			Collection<IMetaModelDescriptor> resourceVersions = safeRelease.getCompatibleResourceVersionDescriptors();
			if (valueAsObject != safeRelease && !resourceVersions.contains(valueAsObject)) {
				throw new RuntimeException("Resource version " //$NON-NLS-1$
						+ valueAsObject.getIdentifier() + " is not compatible with SAFE release " + safeRelease.getName() //$NON-NLS-1$
						+ " set in project " //$NON-NLS-1$
						+ project.getName() + "."); //$NON-NLS-1$
			}

			return valueAsObject.getIdentifier();
		}
	};
}